from collections.abc import Sequence
import numpy as np


#PV data type: enum, string, char, float or int
DTYPES = {
    str: "string",
    float: "float",
    int: "int"
}

STR_REPS = {v:k for k, v in DTYPES.items()}

MAX_STRING_LENGTH = 40 # epics string is limited to 40 characters, beyond that char arrays are needed



class PVInfo:

    def __init__(self, name, data, parse_string=True):
        self.name = name.upper()
        #TODO: the following is ugly
        res = infer_type(data, parse_string=parse_string)
        if len(res) == 2:
            self.dtype, self.value = res
            self.count = 1
        elif len(res) == 3:
            self.dtype, self.value, self.count = res

    def to_dict(self):
        #TODO: the following is ugly
        if self.count == 1:
            return {"type": self.dtype}
        else:
            return {"type": self.dtype, "count": self.count}



def infer_type(value, parse_string=True):
    if isinstance(value, Sequence) and not isinstance(value, str):
        value = np.array(value) # rely on numpy to figure out a consistent dtype

    if isinstance(value, np.ndarray):
        return infer_type_numpy(value) #TODO: also parse strings for np arrays?!

    if isinstance(value, np.generic): # covers all numpy scalars
        value = value.item()

    if parse_string and isinstance(value, str):
        if value in STR_REPS:
            value = STR_REPS[value]

    if isinstance(value, str):
        return infer_type_str(value)

    return infer_type_scalar(value)



def infer_type_scalar(value):
    if isinstance(value, type):
        dtype = value
        value = dtype()
    else:
        dtype = type(value)

    #TODO: bool -> int or string?
    if isinstance(value, bool):
        dtype = int
        value = int(value)

    if dtype in DTYPES:
        dtype = DTYPES[dtype]
    else:
        # if dtype not understood, default to string and use string representation
        dtype = "string"
        value = str(value)

    return dtype, value



def infer_type_numpy(arr):
    # cannot infer a single type from object arrays, default to string representation
    if arr.dtype == object:
        return "string", str(arr.tolist())

    # bool -> int
    if arr.dtype == bool:
        arr = arr.astype(int)

    # epics only allows 1D, and needs conversion to built-in data types
    lst = arr.ravel().tolist()
    dtype, _ = infer_type_scalar(lst[0]) # TODO: is taking the first element safe? object arrays are ruled out above...
    count = len(lst)

    return dtype, lst, count



def infer_type_str(value):
    count = len(value)
    #TODO: handle count == 1 as a single char?
    if count > MAX_STRING_LENGTH:
        return "char", value, count
    else:
        return "string", value



