#!/usr/bin/env python

from functools import partial
from math import nan
import numpy as np
import pytest


from .pvinfo import infer_type as it

psfalse = partial(it, parse_string=False)
pstrue  = partial(it, parse_string=True)


def test_it_type_str():
    assert it(str) == ("string", "")

def test_it_type_float():
    assert it(float) == ("float", 0.0)

def test_it_type_int():
    assert it(int) == ("int", 0)


def test_it_value_str():
    assert it("test") == ("string", "test")

def test_it_value_long_str():
    dat = "test" * 100
    ref = dat
    assert it(dat) == ("char", ref, len(ref))

def test_it_value_float():
    assert it(1.23) == ("float", 1.23)

def test_it_value_int():
    assert it(123) == ("int", 123)


def test_it_empty_value_str():
    assert it("") == ("string", "")

def test_it_empty_value_float():
    assert it(0.0) == ("float", 0.0)

def test_it_empty_value_int():
    assert it(0) == ("int", 0)


def test_pstrue_str():
    assert pstrue("string") == ("string", "")

def test_pstrue_float():
    assert pstrue("float") == ("float", 0.0)

def test_pstrue_int():
    assert pstrue("int") == ("int", 0)


def test_psfalse_str():
    assert psfalse("string") == ("string", "string")

def test_psfalse_float():
    assert psfalse("float") == ("string", "float")

def test_psfalse_int():
    assert psfalse("int") == ("string", "int")


def test_it_None():
    assert it(None) == ("string", "None")

def test_it_True():
    assert it(True) == ("int", 1)

def test_it_False():
    assert it(False) == ("int", 0)

def test_it_nan():
    assert it(nan) == ("float", nan)

def test_it_np_nan():
    assert it(np.nan) == ("float", np.nan)


def test_it_np1D_int():
    n = 10
    arr = np.arange(n)
    ref = arr.tolist()
    assert it(arr) == ("int", ref, n)

def test_it_np2D_int():
    x = y = 10
    n = x * y
    arr = np.arange(n)
    ref = arr.tolist()
    arr = arr.reshape(x, y)
    assert it(arr) == ("int", ref, n)

def test_it_np1D_float():
    n = 10
    arr = np.linspace(0.1, 23.4, n)
    ref = arr.tolist()
    assert it(arr) == ("float", ref, n)

def test_it_np2D_float():
    x = y = 10
    n = x * y
    arr = np.linspace(0.1, 23.4, n)
    ref = arr.tolist()
    arr = arr.reshape(x, y)
    assert it(arr) == ("float", ref, n)


def test_it_np1D_bool():
    arr = np.array([True, False, True])
    ref = [1, 0, 1]
    assert it(arr) == ("int", ref, len(ref))

def test_it_np1D_object():
    arr = np.array([None]) # gives dtype=object
    assert it(arr) == ("string", "[None]")
    arr = np.array(["test", 123, 1.23], dtype=object)
    assert it(arr) == ("string", "['test', 123, 1.23]")


def test_it_np_scalar_int():
    scalar = np.array(123)[()]
    ref = scalar.item()
    assert it(scalar) == ("int", ref)

def test_it_np_scalar_float():
    scalar = np.array(1.23)[()]
    ref = scalar.item()
    assert it(scalar) == ("float", ref)

def test_it_np_scalar_bool():
    scalar = np.array(True)[()]
    ref = scalar.item()
    assert it(scalar) == ("int", 1)
    scalar = np.array(False)[()]
    ref = scalar.item()
    assert it(scalar) == ("int", 0)


def test_it_list():
    dat = [0, 1, 2]
    ref = dat
    assert it(dat) == ("int", ref, len(ref))

def test_it_tuple():
    dat = (0, 1, 2)
    ref = list(dat)
    assert it(dat) == ("int", ref, len(ref))



